# -*- coding: utf-8 -*-
# This code is part of QKDSimkit.
#
# SPDX-License-Identifier: MIT
#
# (C) Copyright 2021 CERN.


class qsocketerror(Exception):
    pass

class qobjecterror(Exception):
    pass
